/*
 * Copyright © 2023 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see
 * <https://www.gnu.org/licenses/>.
 *
 * Additionally, permission is granted to copy, distribute and/or
 * modify this document under the terms of the GNU Free Documentation
 * License, Version 1.3 or any later version published by the Free
 * Software Foundation; with no Invariant Sections, no Front-Cover
 * Texts, and no Back-Cover Texts. A copy of the license is included
 * in the section entitled "GNU Free Documentation License".
 */
#include <mbenchmark/all.hpp>
#include <chrono>
#include <iostream>
#include <thread>

class test_functor
{
  using ms = std::chrono::milliseconds;
public:
  static void
  constraints (micro_benchmark::test_case& test)
  {
    test.add_dimension ({100, 200, 300});
  }

  ms
  set_up (micro_benchmark::state const& v) const
  {
    auto sizes = v.sizes ();
    return ms{sizes.front ()};
  }

  void
  tear_down (micro_benchmark::state const&, ms const& s) const
  {
    std::cout << "Tear down test with " << s.count () << "ms\n";
  }

  void
  operator() (ms t) const
  {
    std::this_thread::sleep_for (t);
  }
} fun;

auto rfunctor = micro_benchmark::register_class("functor", fun);

class test_void
{
  using ms = std::chrono::milliseconds;
  ms value = ms{1000};
public:
  static void
  constraints (micro_benchmark::test_case& test)
  {
    test.add_dimension ({100, 200, 300});
  }

  void
  set_up (micro_benchmark::state const& v)
  {
    auto sizes = v.sizes ();
    value = ms{sizes.front ()};
  }

  void
  tear_down (micro_benchmark::state const&)
  {
    std::cout << "Tear down test with " << value.count () << "ms\n";
  }

  void
  operator() () const
  {
    std::this_thread::sleep_for (value);
  }
} object;

auto rvoid = micro_benchmark::register_class("void", object);

MICRO_BENCHMARK_MAIN ();
