/* xalloc.h --- Abort when there is no memory. */
/*
 * Copyright © 2023 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
 *
 * This file is part of MicroBenchmark.
 *
 * MicroBenchmark is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * MicroBenchmark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with MicroBenchmark.  If not, see
 * <http://www.gnu.org/licenses/>.
 */
#ifndef MICRO_BENCHMARK_INTERNAL_UTILITY_XALLOC_H
#define MICRO_BENCHMARK_INTERNAL_UTILITY_XALLOC_H

#include <stddef.h>

#include "mbenchmark/common.h"

/* *INDENT-OFF* */
MICRO_BENCHMARK_BEGIN_DECLS
/* *INDENT-ON* */

#if defined(__GNUC__) && defined(__SANITIZE_ADDRESS__)
#include <stdlib.h>
/*
  This produces better traces instead of always pointing to the
  allocation functions as only responsible for the allocation.
*/
#define xmalloc(x) malloc(x)
#define xmalloc_n(x, n) malloc(x * n)
#define xcalloc(x) calloc(1, x)
#define xcalloc_n(x, n) calloc(n, x)
#define xrealloc_n(p, x, n) realloc(p, x * n)
#define xfree(p, x) free(p)
#define xfree_n(p, x, n) free(p)
#else
/** Memory allocation with error checking.  */
void *xmalloc (size_t size);

/** Array allocation with error checking.  */
void *xmalloc_n (size_t size, size_t n);

/** Zeroed object allocation with error checking.  */
void *xcalloc (size_t size);

/** Zeroed array allocation with error checking.  */
void *xcalloc_n (size_t size, size_t n);

/** Array reallocation with error checking.  */
void *xrealloc_n (void *ptr, size_t size, size_t n);

/** Relase allocated memory.  */
void xfree (void *ptr, size_t size);

/** Relase allocated array.  */
void xfree_n (void *ptr, size_t size, size_t n);
#endif

MICRO_BENCHMARK_END_DECLS
#endif
